@@ -19,6 +19,7 @@ from math import atan2, cos, pi, sin, tan
 from warnings import warn
 
 from igraph._igraph import convex_hull, VertexSeq
+from igraph.compat import property
 from igraph.configuration import Configuration
 from igraph.drawing.baseclasses import AbstractDrawer, AbstractCairoDrawer, \
                                        AbstractXMLRPCDrawer
@@ -191,6 +192,8 @@ class DefaultGraphDrawer(AbstractCairoGraphDrawer):
             """Collects some visual properties of a vertex for drawing"""
             _kwds_prefix = "vertex_"
             color = ("red", palette.get)
+            frame_color = ("black", palette.get)
+            frame_width = 1.0
             label = None
             label_angle = -pi/2
             label_dist  = 0.0
@@ -321,7 +324,8 @@ class DefaultGraphDrawer(AbstractCairoGraphDrawer):
                     coords[0], coords[1], vertex.size)
             context.set_source_rgba(*vertex.color)
             context.fill_preserve()
-            context.set_source_rgb(0., 0., 0.)
+            context.set_source_rgba(*vertex.frame_color)
+            context.set_line_width(vertex.frame_width)
             context.stroke()
 
         # Draw the vertex labels
@@ -776,4 +780,61 @@ class CytoscapeGraphDrawer(AbstractXMLRPCDrawer, AbstractGraphDrawer):
 
 #####################################################################
 
+class GephiGraphStreamingDrawer(AbstractGraphDrawer):
+    """Graph drawer that sends a graph to a file-like object (e.g., socket, URL
+    connection, file) using the Gephi graph streaming format.
+
+    The Gephi graph streaming format is a simple JSON-based format that can be used
+    to post mutations to a graph (i.e. node and edge additions, removals and updates)
+    to a remote component. For instance, one can open up Gephi (U{http://www.gephi.org}),
+    install the Gephi graph streaming plugin and then send a graph from igraph
+    straight into the Gephi window by using C{GephiGraphStreamingDrawer} with the
+    appropriate URL where Gephi is listening.
+
+    The C{connection} property exposes the L{GephiConnection} that the drawer
+    uses. The drawer also has a property called C{streamer} which exposes the underlying
+    L{GephiGraphStreamer} that is responsible for generating the JSON objects,
+    encoding them and writing them to a file-like object. If you want to customize
+    the encoding process, this is the object where you can tweak things to your taste.
+    """
+
+    def __init__(self, conn=None, *args, **kwds):
+        """Constructs a Gephi graph streaming drawer that will post graphs to the
+        given Gephi connection. If C{conn} is C{None}, the remaining arguments of
+        the constructor are forwarded intact to the constructor of
+        L{GephiConnection} in order to create a connection. This means that any of
+        the following are valid:
+
+          - C{GephiGraphStreamingDrawer()} will construct a drawer that connects to
+            workspace 0 of the local Gephi instance on port 8080.
+
+          - C{GephiGraphStreamingDrawer(workspace=2)} will connect to workspace 2
+            of the local Gephi instance on port 8080.
+
+          - C{GephiGraphStreamingDrawer(port=1234)} will connect to workspace 0
+            of the local Gephi instance on port 1234.
+
+          - C{GephiGraphStreamingDrawer(host="remote", port=1234, workspace=7)}
+            will connect to workspace 7 of the Gephi instance on host C{remote},
+            port 1234.
+
+          - C{GephiGraphStreamingDrawer(url="http://remote:1234/workspace7)} is
+            the same as above, but with an explicit URL.
+        """
+        super(GephiGraphStreamingDrawer, self).__init__()
+
+        from igraph.remote.gephi import GephiGraphStreamer, GephiConnection
+        self.connection = conn or GephiConnection(*args, **kwds)
+        self.streamer = GephiGraphStreamer()
+
+    def draw(self, graph, *args, **kwds):
+        """Draws (i.e. sends) the given graph to the destination of the drawer using
+        the Gephi graph streaming API.
+        
+        The following keyword arguments are allowed:
+            
+            - ``encoder`` lets one specify an instance of ``json.JSONEncoder`` that
+              will be used to encode the JSON objects.
+        """
+        self.streamer.post(graph, self.connection, encoder=kwds.get("encoder"))
 
