@@ -2,7 +2,7 @@ import unittest
 from igraph.test import basic, layouts, games, foreign, structural, flow, \
     spectral, attributes, cliques, decomposition, operators, generators, \
     isomorphism, colortests, vertexseq, edgeseq, iterators, bipartite, \
-    conversion, rng, separators, indexing, atlas, matching
+    conversion, rng, separators, indexing, atlas, matching, homepage
 
 def suite():
     return unittest.TestSuite( \
@@ -29,7 +29,8 @@ def suite():
          separators.suite(),
          indexing.suite(),
          atlas.suite(),
-         matching.suite()] \
+         matching.suite(),
+         homepage.suite()] \
     )
     
 def run_tests(verbosity=1):
