@@ -24,15 +24,25 @@ class OperatorTests(unittest.TestCase):
         self.failUnless(g.vcount() == 5 and g.ecount() == 3
                         and g.clusters().membership == [0,0,0,1,2])
 
+        # Adding a vertex by name
+        g += "spam"
+        self.failUnless(g.vcount() == 6 and g.ecount() == 3
+                        and g.clusters().membership == [0,0,0,1,2,3])
+
         # Adding a single edge
         g += (2, 3)
-        self.failUnless(g.vcount() == 5 and g.ecount() == 4
-                        and g.clusters().membership == [0,0,0,0,1])
+        self.failUnless(g.vcount() == 6 and g.ecount() == 4
+                        and g.clusters().membership == [0,0,0,0,1,2])
 
         # Adding two edges
-        g += [(3, 4), (2, 4)]
-        self.failUnless(g.vcount() == 5 and g.ecount() == 6
-                        and g.clusters().membership == [0]*5)
+        g += [(3, 4), (2, 4), (4, 5)]
+        self.failUnless(g.vcount() == 6 and g.ecount() == 7
+                        and g.clusters().membership == [0]*6)
+
+        # Adding two more vertices
+        g += ["eggs", "bacon"]
+        self.assertEquals(g.vs["name"], [None, None, None, None, None,
+            "spam", "eggs", "bacon"])
 
         # Did we really use the original graph so far?
         # TODO: disjoint union should be modified so that this assertion
@@ -41,13 +51,13 @@ class OperatorTests(unittest.TestCase):
 
         # Adding another graph
         g += Graph.Full(3)
-        self.failUnless(g.vcount() == 8 and g.ecount() == 9
-                        and g.clusters().membership == [0,0,0,0,0,1,1,1])
+        self.failUnless(g.vcount() == 11 and g.ecount() == 10
+                        and g.clusters().membership == [0,0,0,0,0,0,1,2,3,3,3])
 
         # Adding two graphs
         g += [Graph.Full(3), Graph.Full(2)]
-        self.failUnless(g.vcount() == 13 and g.ecount() == 13
-                        and g.clusters().membership == [0,0,0,0,0,1,1,1,2,2,2,3,3])
+        self.failUnless(g.vcount() == 16 and g.ecount() == 14
+                        and g.clusters().membership == [0,0,0,0,0,0,1,2,3,3,3,4,4,4,5,5])
 
     def testAddition(self):
         g0 = Graph.Full(3)
@@ -58,22 +68,28 @@ class OperatorTests(unittest.TestCase):
                         and g.clusters().membership == [0,0,0,1,2])
         g0 = g
 
+        # Adding vertices by name
+        g = g0+"spam"
+        self.failUnless(g.vcount() == 6 and g.ecount() == 3
+                        and g.clusters().membership == [0,0,0,1,2,3])
+        g0 = g
+
         # Adding a single edge
         g = g0+(2,3)
-        self.failUnless(g.vcount() == 5 and g.ecount() == 4
-                        and g.clusters().membership == [0,0,0,0,1])
+        self.failUnless(g.vcount() == 6 and g.ecount() == 4
+                        and g.clusters().membership == [0,0,0,0,1,2])
         g0 = g
 
         # Adding two edges
-        g = g0+[(3, 4), (2, 4)]
-        self.failUnless(g.vcount() == 5 and g.ecount() == 6
-                        and g.clusters().membership == [0]*5)
+        g = g0+[(3, 4), (2, 4), (4, 5)]
+        self.failUnless(g.vcount() == 6 and g.ecount() == 7
+                        and g.clusters().membership == [0]*6)
         g0 = g
 
         # Adding another graph
         g = g0+Graph.Full(3)
-        self.failUnless(g.vcount() == 8 and g.ecount() == 9
-                        and g.clusters().membership == [0,0,0,0,0,1,1,1])
+        self.failUnless(g.vcount() == 9 and g.ecount() == 10
+                        and g.clusters().membership == [0,0,0,0,0,0,1,1,1])
 
     def testInPlaceSubtraction(self):
         g = Graph.Full(8)
@@ -113,6 +129,14 @@ class OperatorTests(unittest.TestCase):
         self.failUnless(g.vcount() == 4 and g.ecount() == 1
                         and g.clusters().membership == [0,1,2,2])
 
+    def testNonzero(self):
+        self.failUnless(Graph(1))
+        self.failIf(Graph(0))
+
+    def testLength(self):
+        self.assertRaises(TypeError, len, Graph(15))
+        self.failUnless(len(Graph(15).vs) == 15)
+        self.failUnless(len(Graph.Full(5).es) == 10)
 
     def testSimplify(self):
         el = [(0,1), (1,0), (1,2), (2,3), (2,3), (2,3), (3,3)] 
@@ -134,35 +158,29 @@ class OperatorTests(unittest.TestCase):
         self.failUnless(g2.vcount() == g.vcount())
         self.failUnless(g2.ecount() == g.ecount() - 1)
 
-    def testSimplifyAttributes(self):
-        el = [(0,1), (1,0), (1,2), (2,3), (2,3), (2,3), (3,3)] 
-        g = Graph(el)
-        g.es["weight"] = [1, 2, 3, 4, 5, 6, 7]
-        g.es["weight2"] = [1, 2, 3, 4, 5, 6, 7]
+    def testContractVertices(self):
+        g = Graph.Full(4) + Graph.Full(4) + [(0, 5), (1, 4)]
 
         g2 = g.copy()
-        g2.simplify(reduce_attributes=max)
-        self.failUnless(g2.es["weight"] == [2, 3, 6])
-        self.failUnless(g2.es["weight2"] == [2, 3, 6])
+        g2.contract_vertices([0, 1, 2, 3, 1, 0, 4, 5])
+        self.assertEquals(g2.vcount(), 6)
+        self.assertEquals(g2.ecount(), g.ecount())
+        self.assertEquals(sorted(g2.get_edgelist()),
+                [(0, 0), (0, 1), (0, 1), (0, 2), (0, 3), (0, 4), (0, 5),
+                 (1, 1), (1, 2), (1, 3), (1, 4), (1, 5), (2, 3), (4, 5)])
 
         g2 = g.copy()
-        g2.simplify(reduce_attributes={"weight": max})
-        self.failUnless(g2.es["weight"] == [2, 3, 6])
-
-        g2 = g.copy()
-        g2.simplify(reduce_attributes={"weight": max, "weight2": sum})
-        self.failUnless(g2.es["weight"] == [2, 3, 6])
-        self.failUnless(g2.es["weight2"] == [3, 3, 15])
-
-        g = Graph(el, directed=True)
-        g.es["weight"] = [1, 2, 3, 4, 5, 6, 7]
-        g.es["weight2"] = [1, 2, 3, 4, 5, 6, 7]
-
-        g2 = g.copy()
-        g2.simplify(reduce_attributes={"weight": max, "weight2": sum})
-        self.failUnless(g2.es["weight"] == [1, 2, 3, 6])
-        self.failUnless(g2.es["weight2"] == [1, 2, 3, 15])
+        g2.contract_vertices([0, 1, 2, 3, 1, 0, 6, 7])
+        self.assertEquals(g2.vcount(), 8)
+        self.assertEquals(g2.ecount(), g.ecount())
+        self.assertEquals(sorted(g2.get_edgelist()),
+                [(0, 0), (0, 1), (0, 1), (0, 2), (0, 3), (0, 6), (0, 7),
+                 (1, 1), (1, 2), (1, 3), (1, 6), (1, 7), (2, 3), (6, 7)])
 
+        g2 = Graph(10)
+        g2.contract_vertices([0, 0, 1, 1, 2, 2, 3, 3, 4, 4])
+        self.assertEquals(g2.vcount(), 5)
+        self.assertEquals(g2.ecount(), 0)
 
 
 def suite():
