@@ -1,3 +1,4 @@
+# vim:set ts=4 sw=4 sts=4 et:
 import unittest
 from igraph import *
 
@@ -10,12 +11,18 @@ class SpectralTests(unittest.TestCase):
 
     def testLaplacian(self):
         g=Graph.Full(3)
+        g.es["weight"] = [1, 2, 3]
         self.failUnless(g.laplacian() == [[ 2, -1, -1],\
                                           [-1,  2, -1],\
                                           [-1, -1,  2]])
         self.assertAlmostEqualsMatrix(g.laplacian(normalized=True),
                 [[1, -0.5, -0.5], [-0.5, 1, -0.5], [-0.5, -0.5, 1]])
 
+        mx0 = [[1., -1/(12**0.5), -2/(15**0.5)],
+              [-1/(12**0.5), 1., -3/(20**0.5)],
+              [-2/(15**0.5), -3/(20**0.5), 1.]]
+        self.assertAlmostEqualsMatrix(g.laplacian("weight", True), mx0)
+
         g=Graph.Tree(5, 2)
         g.add_vertices(1)
         self.failUnless(g.laplacian() == [[ 2, -1, -1,  0,  0, 0],\
