@@ -1,8 +1,7 @@
 /* -*- mode: C -*-  */
 /* 
    IGraph library.
-   Copyright (C) 2006  Gabor Csardi <csardi@rmki.kfki.hu>
-   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+   Copyright (C) 2006-2012  Tamas Nepusz <ntamas@gmail.com>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,9 +21,10 @@
 */
 
 #include "bfsiter.h"
-#include "vertexobject.h"
 #include "common.h"
 #include "error.h"
+#include "py2compat.h"
+#include "vertexobject.h"
 
 /**
  * \ingroup python_interface
@@ -159,9 +159,9 @@ PyObject* igraphmodule_BFSIter_iter(igraphmodule_BFSIterObject* self) {
 
 PyObject* igraphmodule_BFSIter_iternext(igraphmodule_BFSIterObject* self) {
   if (!igraph_dqueue_empty(&self->queue)) {
-    long int vid = igraph_dqueue_pop(&self->queue);
-    long int dist = igraph_dqueue_pop(&self->queue);
-    long int parent = igraph_dqueue_pop(&self->queue);
+    igraph_integer_t vid = (igraph_integer_t)igraph_dqueue_pop(&self->queue);
+    igraph_integer_t dist = (igraph_integer_t)igraph_dqueue_pop(&self->queue);
+    igraph_integer_t parent = (igraph_integer_t)igraph_dqueue_pop(&self->queue);
     long int i;
     
     if (igraph_neighbors(self->graph, &self->neis, vid, self->mode)) {
@@ -170,7 +170,7 @@ PyObject* igraphmodule_BFSIter_iternext(igraphmodule_BFSIterObject* self) {
     }
 	
     for (i=0; i<igraph_vector_size(&self->neis); i++) {
-      long int neighbor=VECTOR(self->neis)[i];
+      igraph_integer_t neighbor = (igraph_integer_t)VECTOR(self->neis)[i];
       if (self->visited[neighbor]==0) {
 	self->visited[neighbor]=1;
 	if (igraph_dqueue_push(&self->queue, neighbor) ||
@@ -183,23 +183,22 @@ PyObject* igraphmodule_BFSIter_iternext(igraphmodule_BFSIterObject* self) {
     }
 
     if (self->advanced) {
-      PyObject *vertexobj, *parentobj, *result;
-      vertexobj = igraphmodule_Vertex_New(self->gref, (long)vid);
-      if (!vertexobj) return NULL;
-      if (parent>=0) {
-	parentobj = igraphmodule_Vertex_New(self->gref, (long)parent);
-	if (!parentobj) return NULL;
+      PyObject *vertexobj, *parentobj;
+      vertexobj = igraphmodule_Vertex_New(self->gref, vid);
+      if (!vertexobj)
+        return NULL;
+      if (parent >= 0) {
+        parentobj = igraphmodule_Vertex_New(self->gref, parent);
+        if (!parentobj)
+            return NULL;
       } else {
-	Py_INCREF(Py_None);
-	parentobj=Py_None;
+        Py_INCREF(Py_None);
+        parentobj=Py_None;
       }
-      result=PyTuple_New(3);
-      PyTuple_SetItem(result, 0, vertexobj);
-      PyTuple_SetItem(result, 1, PyInt_FromLong(dist));
-      PyTuple_SetItem(result, 2, parentobj);
-      return result;
-    } else
-      return igraphmodule_Vertex_New(self->gref, (long)vid);
+      return Py_BuildValue("NlN", vertexobj, (long int)dist, parentobj);
+    } else {
+      return igraphmodule_Vertex_New(self->gref, vid);
+    }
   } else {
     return NULL;
   }
@@ -219,8 +218,7 @@ PyMethodDef igraphmodule_BFSIter_methods[] = {
  */
 PyTypeObject igraphmodule_BFSIterType =
 {
-  PyObject_HEAD_INIT(NULL)                  //
-  0,                                        // ob_size
+  PyVarObject_HEAD_INIT(0, 0)
   "igraph.BFSIter",                         // tp_name
   sizeof(igraphmodule_BFSIterObject),       // tp_basicsize
   0,                                        // tp_itemsize
@@ -228,7 +226,7 @@ PyTypeObject igraphmodule_BFSIterType =
   0,                                        // tp_print
   0,                                        // tp_getattr
   0,                                        // tp_setattr
-  0,                                        // tp_compare
+  0,                                        /* tp_compare (2.x) / tp_reserved (3.x) */
   0,                                        // tp_repr
   0,                                        // tp_as_number
   0,                                        // tp_as_sequence
