@@ -3,8 +3,7 @@
 
 /* 
    IGraph library.
-   Copyright (C) 2006  Gabor Csardi <csardi@rmki.kfki.hu>
-   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+   Copyright (C) 2006-2012  Tamas Nepusz <ntamas@gmail.com>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,9 +22,12 @@
 
 */
 
+#include "attributes.h"
 #include "edgeobject.h"
 #include "graphobject.h"
+#include "vertexobject.h"
 #include "error.h"
+#include "py2compat.h"
 
 /**
  * \ingroup python_interface
@@ -46,7 +48,7 @@ PyTypeObject igraphmodule_EdgeType;
  * changes, your existing edge objects will point to elsewhere
  * (or they might even get invalidated).
  */
-PyObject* igraphmodule_Edge_New(igraphmodule_GraphObject *gref, long idx) {
+PyObject* igraphmodule_Edge_New(igraphmodule_GraphObject *gref, igraph_integer_t idx) {
   igraphmodule_EdgeObject* self;
   self=PyObject_New(igraphmodule_EdgeObject, &igraphmodule_EdgeType);
   if (self) {
@@ -90,23 +92,41 @@ void igraphmodule_Edge_dealloc(igraphmodule_EdgeObject* self) {
  * \return the formatted textual representation as a \c PyObject
  */
 PyObject* igraphmodule_Edge_repr(igraphmodule_EdgeObject *self) {
-  PyObject *s, *grepr, *drepr;
+  PyObject *s;
+  PyObject *attrs;
+#ifndef IGRAPH_PYTHON3
+  PyObject *grepr, *drepr;
+#endif
 
+  attrs = igraphmodule_Edge_attributes(self);
+  if (attrs == 0)
+    return NULL;
+
+#ifdef IGRAPH_PYTHON3
+  s = PyUnicode_FromFormat("igraph.Edge(%R, %ld, %R)",
+      (PyObject*)self->gref, (long int)self->idx, attrs);
+  Py_DECREF(attrs);
+#else
   grepr=PyObject_Repr((PyObject*)self->gref);
-  if (!grepr) return NULL;
-  drepr=PyObject_Repr(igraphmodule_Edge_attributes(self));
-  if (!drepr) { Py_DECREF(grepr); return NULL; }
-  s=PyString_FromFormat("igraph.Edge(%s,%ld,%s)", PyString_AsString(grepr),
-    self->idx, PyString_AsString(drepr));
+  drepr=PyObject_Repr(attrs);
+  Py_DECREF(attrs);
+  if (!grepr || !drepr) {
+    Py_XDECREF(grepr);
+    Py_XDECREF(drepr);
+    return NULL;
+  }
+  s=PyString_FromFormat("igraph.Edge(%s, %ld, %s)", PyString_AsString(grepr),
+    (long int)self->idx, PyString_AsString(drepr));
   Py_DECREF(grepr);
   Py_DECREF(drepr);
+#endif
   return s;
 }
 
 /** \ingroup python_interface_edge
  * \brief Returns the number of edge attributes
  */
-int igraphmodule_Edge_attribute_count(igraphmodule_EdgeObject* self) {
+Py_ssize_t igraphmodule_Edge_attribute_count(igraphmodule_EdgeObject* self) {
   igraphmodule_GraphObject *o = self->gref;
   
   if (!o) return 0;
@@ -128,15 +148,19 @@ PyObject* igraphmodule_Edge_attribute_names(igraphmodule_EdgeObject* self) {
 PyObject* igraphmodule_Edge_attributes(igraphmodule_EdgeObject* self) {
   igraphmodule_GraphObject *o = self->gref;
   PyObject *names, *dict;
-  long i, n;
+  long int i, n;
 
   dict=PyDict_New();
-  if (!dict) return NULL;
+  if (!dict)
+    return NULL;
 
   names=igraphmodule_Graph_edge_attributes(o);
-  if (!names) { Py_DECREF(dict); return NULL; }
+  if (!names) {
+    Py_DECREF(dict);
+    return NULL;
+  }
 
-  n=PyList_Size(names);
+  n = PyList_Size(names);
   for (i=0; i<n; i++) {
     PyObject *name = PyList_GetItem(names, i);
     if (name) {
@@ -157,21 +181,17 @@ PyObject* igraphmodule_Edge_attributes(igraphmodule_EdgeObject* self) {
 
 /**
  * \ingroup python_interface_edge
- * Returns the edge index
+ * \brief Updates some attributes of an edge
+ * 
+ * \param self the edge object
+ * \param args positional arguments
+ * \param kwds keyword arguments
  */
-PyObject* igraphmodule_Edge_get_index(igraphmodule_EdgeObject* self, void* closure) {
-  return PyInt_FromLong((long)self->idx);
+PyObject* igraphmodule_Edge_update_attributes(PyObject* self, PyObject* args,
+    PyObject* kwds) {
+  return igraphmodule_Vertex_update_attributes(self, args, kwds);
 }
 
-/**
- * \ingroup python_interface_edge
- * Returns the edge index as an ordinary C long
- */
-long igraphmodule_Edge_get_index_long(igraphmodule_EdgeObject* self) {
-  return (long)self->idx;
-}
-
-
 /** \ingroup python_interface_edge
  * \brief Returns the corresponding value to a given attribute of the edge
  * \param self the edge object
@@ -279,7 +299,7 @@ PyObject* igraphmodule_Edge_get_from(igraphmodule_EdgeObject* self, void* closur
   if (igraph_edge(&o->g, self->idx, &from, &to)) {
     igraphmodule_handle_igraph_error(); return NULL;
   }
-  return PyInt_FromLong((long)from);
+  return PyInt_FromLong((long int)from);
 }
 
 /**
@@ -296,6 +316,30 @@ PyObject* igraphmodule_Edge_get_to(igraphmodule_EdgeObject* self, void* closure)
   return PyInt_FromLong((long)to);
 }
 
+/**
+ * \ingroup python_interface_edge
+ * Returns the edge index
+ */
+PyObject* igraphmodule_Edge_get_index(igraphmodule_EdgeObject* self, void* closure) {
+  return PyInt_FromLong((long int)self->idx);
+}
+
+/**
+ * \ingroup python_interface_edge
+ * Returns the edge index as an igraph_integer_t
+ */
+igraph_integer_t igraphmodule_Edge_get_index_igraph_integer(igraphmodule_EdgeObject* self) {
+  return self->idx;
+}
+
+/**
+ * \ingroup python_interface_edge
+ * Returns the edge index as an ordinary C long
+ */
+long igraphmodule_Edge_get_index_long(igraphmodule_EdgeObject* self) {
+  return (long)self->idx;
+}
+
 /**
  * \ingroup python_interface_edge
  * Returns the target node index of an edge
@@ -310,6 +354,48 @@ PyObject* igraphmodule_Edge_get_tuple(igraphmodule_EdgeObject* self, void* closu
   return Py_BuildValue("(ii)", (long)from, (long)to);
 }
 
+#define GRAPH_PROXY_METHOD(FUNC, METHODNAME) \
+    PyObject* igraphmodule_Edge_##FUNC(igraphmodule_EdgeObject* self, PyObject* args, PyObject* kwds) { \
+      PyObject *new_args, *item, *result;                     \
+      long int i, num_args = args ? PyTuple_Size(args)+1 : 1; \
+                                                              \
+      /* Prepend ourselves to args */                         \
+      new_args = PyTuple_New(num_args);                       \
+      Py_INCREF(self); PyTuple_SET_ITEM(new_args, 0, (PyObject*)self);   \
+      for (i = 1; i < num_args; i++) {                        \
+        item = PyTuple_GET_ITEM(args, i-1);                   \
+        Py_INCREF(item); PyTuple_SET_ITEM(new_args, i, item); \
+      }                                                       \
+                                                              \
+      /* Get the method instance */                           \
+      item = PyObject_GetAttrString((PyObject*)(self->gref), METHODNAME);  \
+      result = PyObject_Call(item, new_args, kwds);           \
+      Py_DECREF(item);                                        \
+      Py_DECREF(new_args);                                    \
+      return result;                                          \
+    }
+
+GRAPH_PROXY_METHOD(count_multiple, "count_multiple");
+GRAPH_PROXY_METHOD(delete, "delete_edges");
+GRAPH_PROXY_METHOD(is_loop, "is_loop");
+GRAPH_PROXY_METHOD(is_multiple, "is_multiple");
+GRAPH_PROXY_METHOD(is_mutual, "is_mutual");
+
+#undef GRAPH_PROXY_METHOD
+
+#define GRAPH_PROXY_METHOD_SPEC(FUNC, METHODNAME) \
+  {METHODNAME, (PyCFunction)igraphmodule_Edge_##FUNC, METH_VARARGS | METH_KEYWORDS, \
+    "Proxy method to L{Graph." METHODNAME "()}\n\n"              \
+    "This method calls the " METHODNAME " method of the L{Graph} class " \
+    "with this edge as the first argument, and returns the result.\n\n"\
+    "@see: Graph." METHODNAME "() for details."}
+#define GRAPH_PROXY_METHOD_SPEC_2(FUNC, METHODNAME, METHODNAME_IN_GRAPH) \
+  {METHODNAME, (PyCFunction)igraphmodule_Edge_##FUNC, METH_VARARGS | METH_KEYWORDS, \
+    "Proxy method to L{Graph." METHODNAME_IN_GRAPH "()}\n\n"              \
+    "This method calls the " METHODNAME_IN_GRAPH " method of the L{Graph} class " \
+    "with this edge as the first argument, and returns the result.\n\n"\
+    "@see: Graph." METHODNAME_IN_GRAPH "() for details."}
+
 /**
  * \ingroup python_interface_edge
  * Method table for the \c igraph.Edge object
@@ -325,9 +411,27 @@ PyMethodDef igraphmodule_Edge_methods[] = {
       "attribute_names() -> list\n\n"
       "Returns the list of edge attribute names\n"
   },
+  {"update_attributes", (PyCFunction)igraphmodule_Edge_update_attributes,
+    METH_VARARGS | METH_KEYWORDS,
+    "update_attributes(E, **F) -> None\n\n"
+    "Updates the attributes of the edge from dict/iterable E and F.\n\n"
+    "If E has a C{keys()} method, it does: C{for k in E: self[k] = E[k]}.\n"
+    "If E lacks a C{keys()} method, it does: C{for (k, v) in E: self[k] = v}.\n"
+    "In either case, this is followed by: C{for k in F: self[k] = F[k]}.\n\n"
+    "This method thus behaves similarly to the C{update()} method of Python\n"
+    "dictionaries."
+  },
+  GRAPH_PROXY_METHOD_SPEC(count_multiple, "count_multiple"),
+  GRAPH_PROXY_METHOD_SPEC_2(delete, "delete", "delete_edges"),
+  GRAPH_PROXY_METHOD_SPEC(is_loop, "is_loop"),
+  GRAPH_PROXY_METHOD_SPEC(is_multiple, "is_multiple"),
+  GRAPH_PROXY_METHOD_SPEC(is_mutual, "is_mutual"),
   {NULL}
 };
 
+#undef GRAPH_PROXY_METHOD_SPEC
+#undef GRAPH_PROXY_METHOD_SPEC_2
+
 /**
  * \ingroup python_interface_edge
  * Getter/setter table for the \c igraph.Edge object
@@ -368,8 +472,7 @@ PyMappingMethods igraphmodule_Edge_as_mapping = {
  */
 PyTypeObject igraphmodule_EdgeType =
 {
-  PyObject_HEAD_INIT(NULL)                    //
-  0,                                          // ob_size
+  PyVarObject_HEAD_INIT(0, 0)
   "igraph.Edge",                              // tp_name
   sizeof(igraphmodule_EdgeObject),            // tp_basicsize
   0,                                          // tp_itemsize
@@ -377,7 +480,7 @@ PyTypeObject igraphmodule_EdgeType =
   0,                                          // tp_print
   0,                                          // tp_getattr
   0,                                          // tp_setattr
-  0,                                          // tp_compare
+  0,                                          /* tp_compare (2.x) / tp_reserved (3.x) */
   (reprfunc)igraphmodule_Edge_repr,           // tp_repr
   0,                                          // tp_as_number
   0,                                          // tp_as_sequence
@@ -395,8 +498,8 @@ PyTypeObject igraphmodule_EdgeType =
   "(if the edge indices are altered in the original graph).\n\n"
   "The attributes of the edge can be accessed by using the edge\n"
   "as a hash:\n\n"
-  "  >>> e[\"weight\"] = 2\n"
-  "  >>> print e[\"weight\"]\n"
+  "  >>> e[\"weight\"] = 2                  #doctest: +SKIP\n"
+  "  >>> print e[\"weight\"]                #doctest: +SKIP\n"
   "  2\n", // tp_doc
   0,                                          // tp_traverse
   0,                                          // tp_clear
