@@ -3,8 +3,7 @@
 
 /* 
    IGraph library.
-   Copyright (C) 2006  Gabor Csardi <csardi@rmki.kfki.hu>
-   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+   Copyright (C) 2006-2012  Tamas Nepusz <ntamas@gmail.com>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,11 +22,13 @@
 
 */
 
-#include "edgeseqobject.h"
-#include "edgeobject.h"
+#include "attributes.h"
 #include "common.h"
 #include "convert.h"
+#include "edgeseqobject.h"
+#include "edgeobject.h"
 #include "error.h"
+#include "py2compat.h"
 
 #define GET_GRAPH(obj) (((igraphmodule_GraphObject*)obj->gref)->g)
 
@@ -69,7 +70,7 @@ igraphmodule_EdgeSeqObject*
 igraphmodule_EdgeSeq_copy(igraphmodule_EdgeSeqObject* o) {
   igraphmodule_EdgeSeqObject *copy;
 
-  copy=(igraphmodule_EdgeSeqObject*)PyType_GenericNew(o->ob_type, 0, 0);
+  copy=(igraphmodule_EdgeSeqObject*)PyType_GenericNew(Py_TYPE(o), 0, 0);
   if (copy == NULL) return NULL;
  
   if (igraph_es_type(&o->es) == IGRAPH_ES_VECTOR) {
@@ -97,7 +98,7 @@ igraphmodule_EdgeSeq_copy(igraphmodule_EdgeSeqObject* o) {
 
 
 /**
- * \ingroup python_interface_vertexseq
+ * \ingroup python_interface_edgeseq
  * \brief Initialize a new edge sequence object for a given graph
  * \return the initialized PyObject
  */
@@ -116,12 +117,12 @@ int igraphmodule_EdgeSeq_init(igraphmodule_EdgeSeqObject *self,
     igraph_es_all(&es, IGRAPH_EDGEORDER_ID);
   } else if (PyInt_Check(esobj)) {
     /* We selected a single edge */
-    long idx = PyInt_AsLong(esobj);
+    long int idx = PyInt_AsLong(esobj);
     if (idx < 0 || idx >= igraph_ecount(&((igraphmodule_GraphObject*)g)->g)) {
-      PyErr_SetString(PyExc_ValueError, "edge index out of bounds");
+      PyErr_SetString(PyExc_ValueError, "edge index out of range");
       return -1;
     }
-    igraph_es_1(&es, idx);
+    igraph_es_1(&es, (igraph_integer_t)idx);
   } else {
 	/* We selected multiple edges */
     igraph_vector_t v;
@@ -130,7 +131,7 @@ int igraphmodule_EdgeSeq_init(igraphmodule_EdgeSeqObject *self,
       return -1;
     if (!igraph_vector_isininterval(&v, 0, n-1)) {
       igraph_vector_destroy(&v);
-      PyErr_SetString(PyExc_ValueError, "edge index out of bounds");
+      PyErr_SetString(PyExc_ValueError, "edge index out of range");
       return -1;
     }
     if (igraph_es_vector_copy(&es, &v)) {
@@ -160,7 +161,7 @@ void igraphmodule_EdgeSeq_dealloc(igraphmodule_EdgeSeqObject* self) {
     Py_DECREF(self->gref);
     self->gref=0;
   }
-  self->ob_type->tp_free((PyObject*)self);
+  Py_TYPE(self)->tp_free((PyObject*)self);
 
   RC_DEALLOC("EdgeSeq", self);
 }
@@ -188,28 +189,30 @@ int igraphmodule_EdgeSeq_sq_length(igraphmodule_EdgeSeqObject* self) {
 PyObject* igraphmodule_EdgeSeq_sq_item(igraphmodule_EdgeSeqObject* self,
                        Py_ssize_t i) {
   igraph_t *g;
-  long idx = -1;
+  igraph_integer_t idx = -1;
   
   if (!self->gref) return NULL;
   g=&GET_GRAPH(self);
   switch (igraph_es_type(&self->es)) {
     case IGRAPH_ES_ALL:
-      if (i >= 0 && i < (long)igraph_ecount(g)) idx = i;
+      if (i >= 0 && i < igraph_ecount(g))
+        idx = (igraph_integer_t)i;
       break;
 
     case IGRAPH_ES_VECTOR:
     case IGRAPH_ES_VECTORPTR:
       if (i >= 0 && i < igraph_vector_size(self->es.data.vecptr))
-        idx = (long)VECTOR(*self->es.data.vecptr)[i];
+        idx = (igraph_integer_t)VECTOR(*self->es.data.vecptr)[i];
       break;
 
     case IGRAPH_ES_1:
-      if (i == 0) idx = (long)self->es.data.eid;
+      if (i == 0)
+        idx = self->es.data.eid;
       break;
 
     case IGRAPH_ES_SEQ:
       if (i >= 0 && i < self->es.data.seq.to - self->es.data.seq.from)
-        idx = (long)(self->es.data.seq.from + i);
+        idx = self->es.data.seq.from + (igraph_integer_t)i;
       break;
 
     /* TODO: IGRAPH_ES_PAIRS, IGRAPH_ES_ADJ, IGRAPH_ES_PATH,
@@ -231,20 +234,6 @@ PyObject* igraphmodule_EdgeSeq_attribute_names(igraphmodule_EdgeSeqObject* self)
   return igraphmodule_Graph_edge_attributes(self->gref);
 }
 
-/** \ingroup python_interface_edgeseq
- * \brief Returns the count of attribute names
- */
-PyObject* igraphmodule_EdgeSeq_attribute_count(igraphmodule_EdgeSeqObject* self) {
-  PyObject *list;
-  long int size;
-
-  list=igraphmodule_Graph_edge_attributes(self->gref);
-  size=PySequence_Size(list);
-  Py_DECREF(list);
-
-  return Py_BuildValue("i", size);
-}
-
 /** \ingroup python_interface_edgeseq
  * \brief Returns the list of values for a given attribute
  */
@@ -254,7 +243,7 @@ PyObject* igraphmodule_EdgeSeq_get_attribute_values(igraphmodule_EdgeSeqObject*
   long int i, n;
 
   PyErr_Clear();
-  values=PyDict_GetItem(((PyObject**)gr->g.attr)[ATTRHASH_IDX_EDGE], o);
+  values=PyDict_GetItem(ATTR_STRUCT_DICT(&gr->g)[ATTRHASH_IDX_EDGE], o);
   if (!values) {
     PyErr_SetString(PyExc_KeyError, "Attribute does not exist");
     return NULL;
@@ -310,13 +299,19 @@ PyObject* igraphmodule_EdgeSeq_get_attribute_values(igraphmodule_EdgeSeqObject*
   return result;
 }
 
+PyObject* igraphmodule_EdgeSeq_is_all(igraphmodule_EdgeSeqObject* self) {
+  if (igraph_es_is_all(&self->es))
+    Py_RETURN_TRUE;
+  Py_RETURN_FALSE;
+}
+
 PyObject* igraphmodule_EdgeSeq_get_attribute_values_mapping(igraphmodule_EdgeSeqObject *self, PyObject *o) {
   /* Handle integer indices according to the sequence protocol */
   if (PyInt_Check(o))
     return igraphmodule_EdgeSeq_sq_item(self, PyInt_AsLong(o));
 
   /* Handle strings according to the mapping protocol */
-  if (PyString_Check(o) || PyUnicode_Check(o))
+  if (PyBaseString_Check(o))
     return igraphmodule_EdgeSeq_get_attribute_values(self, o);
 
   /* Handle iterables and slices by calling the select() method */
@@ -325,7 +320,7 @@ PyObject* igraphmodule_EdgeSeq_get_attribute_values_mapping(igraphmodule_EdgeSeq
     args = Py_BuildValue("(O)", o);
     if (!args)
       return NULL;
-    result = igraphmodule_EdgeSeq_select(self, args, NULL);
+    result = igraphmodule_EdgeSeq_select(self, args);
     Py_DECREF(args);
     return result;
   }
@@ -341,9 +336,9 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
   PyObject *dict, *list, *item;
   igraphmodule_GraphObject *gr=self->gref;
   igraph_vector_t es;
-  long i, n;
+  long i, j, n, no_of_edges;
   
-  dict = ((PyObject**)gr->g.attr)[ATTRHASH_IDX_EDGE];
+  dict = ATTR_STRUCT_DICT(&gr->g)[ATTRHASH_IDX_EDGE];
 
   if (values == 0) {
     if (igraph_es_type(&self->es) == IGRAPH_ES_ALL)
@@ -352,12 +347,26 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
     return -1;
   }
 
+ if (PyString_Check(values) || !PySequence_Check(values)) {
+    /* If values is a string or not a sequence, we construct a list with a
+     * single element (the value itself) and then call ourselves again */
+    int result;
+    PyObject *newList = PyList_New(1);
+    if (newList == 0) return -1;
+    Py_INCREF(values);
+    PyList_SET_ITEM(newList, 0, values);    /* reference stolen here */
+    result = igraphmodule_EdgeSeq_set_attribute_values_mapping(self, attrname, newList);
+    Py_DECREF(newList);
+    return result;
+  }
+
   n=PySequence_Size(values);
   if (n<0) return -1;
 
   if (igraph_es_type(&self->es) == IGRAPH_ES_ALL) {
-    if (n != (long)igraph_ecount(&gr->g)) {
-      PyErr_SetString(PyExc_ValueError, "value list length must be equal to the number of edges in the edge sequence");
+    no_of_edges = (long)igraph_ecount(&gr->g);
+    if (n == 0 && no_of_edges > 0) {
+      PyErr_SetString(PyExc_ValueError, "sequence must not be empty");
       return -1;
     }
 
@@ -365,8 +374,9 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
     list = PyDict_GetItem(dict, attrname);
     if (list != 0) {
       /* Yes, we have. Modify its items to the items found in values */
-      for (i=0; i<n; i++) {
-        item = PySequence_GetItem(values, i);
+      for (i=0, j=0; i<no_of_edges; i++, j++) {
+        if (j == n) j = 0;
+        item = PySequence_GetItem(values, j);
         if (item == 0) return -1;
         /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         if (PyList_SetItem(list, i, item)) {
@@ -377,10 +387,11 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
     } else if (values != 0) {
       /* We don't have attributes with the given name yet. Create an entry
        * in the dict, create a new list and copy everything */
-      list = PyList_New(n);
+      list = PyList_New(no_of_edges);
       if (list == 0) return -1;
-      for (i=0; i<n; i++) {
-        item = PySequence_GetItem(values, i);
+      for (i=0, j=0; i<no_of_edges; i++, j++) {
+        if (j == n) j = 0;
+        item = PySequence_GetItem(values, j);
         if (item == 0) { Py_DECREF(list); return -1; }
         /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         PyList_SET_ITEM(list, i, item);
@@ -404,8 +415,9 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
       igraph_vector_destroy(&es);
       return -1;
     }
-    if (n != (long)igraph_vector_size(&es)) {
-      PyErr_SetString(PyExc_ValueError, "value list length must be equal to the number of edges in the edge sequence");
+    no_of_edges = (long)igraph_vector_size(&es);
+    if (n == 0 && no_of_edges > 0) {
+      PyErr_SetString(PyExc_ValueError, "sequence must not be empty");
       igraph_vector_destroy(&es);
       return -1;
     }
@@ -413,8 +425,9 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
     list = PyDict_GetItem(dict, attrname);
     if (list != 0) {
       /* Yes, we have. Modify its items to the items found in values */
-      for (i=0; i<n; i++) {
-        item = PySequence_GetItem(values, i);
+      for (i=0, j=0; i<no_of_edges; i++, j++) {
+        if (j == n) j = 0;
+        item = PySequence_GetItem(values, j);
         if (item == 0) { igraph_vector_destroy(&es); return -1; }
         /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         if (PyList_SetItem(list, (long)VECTOR(es)[i], item)) {
@@ -435,8 +448,9 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
         Py_INCREF(Py_None);
         PyList_SET_ITEM(list, i, Py_None);
       }
-      for (i=0; i<n; i++) {
-        item = PySequence_GetItem(values, i);
+      for (i=0, j=0; i<no_of_edges; i++, j++) {
+        if (j == n) j = 0;
+        item = PySequence_GetItem(values, j);
         if (item == 0) {
           igraph_vector_destroy(&es);
           Py_DECREF(list); return -1;
@@ -471,19 +485,61 @@ PyObject* igraphmodule_EdgeSeq_set_attribute_values(igraphmodule_EdgeSeqObject *
   Py_RETURN_NONE;
 }
 
+/**
+ * \ingroup python_interface_edgqseq
+ * \brief Selects a single edge from the edge sequence based on some criteria
+ */
+PyObject* igraphmodule_EdgeSeq_find(igraphmodule_EdgeSeqObject *self, PyObject *args) {
+  PyObject *item;
+  long int i, n;
+
+  if (!PyArg_ParseTuple(args, "O", &item))
+    return NULL;
+
+  if (PyCallable_Check(item)) {
+    /* Call the callable for every edge in the current sequence and return
+     * the first one for which it evaluates to True */
+    n = PySequence_Size((PyObject*)self);
+    for (i=0; i<n; i++) {
+      PyObject *edge = PySequence_GetItem((PyObject*)self, i);
+      PyObject *call_result;
+      if (edge == 0)
+        return NULL;
+      call_result = PyObject_CallFunctionObjArgs(item, edge, NULL);
+      if (call_result == 0) {
+        Py_DECREF(edge);
+        return NULL;
+      }
+      if (PyObject_IsTrue(call_result)) {
+        Py_DECREF(call_result);
+        return edge;  /* reference passed to caller */
+      }
+      Py_DECREF(call_result);
+      Py_DECREF(edge);
+    }
+  } else if (PyInt_Check(item)) {
+    /* Integers are interpreted as indices on the edge set and NOT on the
+     * original, untouched edge sequence of the graph */
+    return PySequence_GetItem((PyObject*)self, PyInt_AsLong(item));
+  }
+
+  PyErr_SetString(PyExc_IndexError, "no such edge");
+  return NULL;
+}
+
 /**
  * \ingroup python_interface_edgeseq
  * \brief Selects a subset of the edge sequence based on some criteria
  */
-PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self,
-  PyObject *args, PyObject *kwds) {
+PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject *args) {
   igraphmodule_EdgeSeqObject *result;
   igraphmodule_GraphObject *gr;
   long i, j, n, m;
 
   gr=self->gref;
   result=igraphmodule_EdgeSeq_copy(self);
-  if (result==0) return NULL;
+  if (result == 0)
+    return NULL;
 
   /* First, filter by positional arguments */
   n = PyTuple_Size(args);
@@ -562,6 +618,7 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self,
         igraphmodule_handle_igraph_error();
         return 0;
       }
+      m = igraph_vector_size(&v2);
       for (; i<n; i++) {
         PyObject *item2 = PyTuple_GET_ITEM(args, i);
         long idx;
@@ -573,6 +630,12 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self,
           return NULL;
         }
         idx = PyInt_AsLong(item2);
+        if (idx >= m || idx < 0) {
+          PyErr_SetString(PyExc_ValueError, "edge index out of range");
+          igraph_vector_destroy(&v);
+          igraph_vector_destroy(&v2);
+          return NULL;
+        }
         if (igraph_vector_push_back(&v, VECTOR(v2)[idx])) {
           Py_DECREF(result);
           igraphmodule_handle_igraph_error();
@@ -611,6 +674,7 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self,
         igraphmodule_handle_igraph_error();
         return 0;
       }
+      m = igraph_vector_size(&v2);
 
       /* Create an appropriate iterator */
       if (PySlice_Check(item)) {
@@ -619,7 +683,9 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self,
         PyObject* range;
         igraph_bool_t ok;
 
-        ok = (PySlice_GetIndicesEx((PySliceObject*)item, igraph_vector_size(&v2),
+        /* Casting to void* because Python 2.x expects PySliceObject*
+         * but Python 3.x expects PyObject* */
+        ok = (PySlice_GetIndicesEx((void*)item, igraph_vector_size(&v2),
               &start, &stop, &step, &sl) == 0);
         if (ok) {
           range = PyObject_CallFunction((PyObject*)&PyRange_Type, "lll", start, stop, step);
@@ -655,6 +721,14 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self,
         if (PyInt_Check(item2)) {
           long idx = PyInt_AsLong(item2);
           Py_DECREF(item2);
+          if (idx >= m || idx < 0) {
+            PyErr_SetString(PyExc_ValueError, "edge index out of range");
+            Py_DECREF(result);
+            Py_DECREF(iter);
+            igraph_vector_destroy(&v);
+            igraph_vector_destroy(&v2);
+            return NULL;
+          }
           if (igraph_vector_push_back(&v, VECTOR(v2)[idx])) {
             Py_DECREF(result);
             Py_DECREF(iter);
@@ -701,12 +775,23 @@ PyMethodDef igraphmodule_EdgeSeq_methods[] = {
    "attribute_names() -> list\n\n"
    "Returns the attribute name list of the graph's edges\n"
   },
+  {"find", (PyCFunction)igraphmodule_EdgeSeq_find,
+   METH_VARARGS,
+   "find(condition) -> Edge\n\n"
+   "For internal use only.\n"
+  },
   {"get_attribute_values", (PyCFunction)igraphmodule_EdgeSeq_get_attribute_values,
    METH_O,
    "get_attribute_values(attrname) -> list\n\n"
    "Returns the value of a given edge attribute for all edges.\n\n"
    "@param attrname: the name of the attribute\n"
   },
+  {"is_all", (PyCFunction)igraphmodule_EdgeSeq_is_all, METH_NOARGS,
+   "is_all() -> bool\n\n"
+   "Returns whether the edge sequence contains all the edges exactly once, in\n"
+   "the order of their edge IDs.\n\n"
+   "This is used for optimizations in some of the edge selector routines.\n"
+  },
   {"set_attribute_values", (PyCFunction)igraphmodule_EdgeSeq_set_attribute_values,
    METH_VARARGS | METH_KEYWORDS,
    "set_attribute_values(attrname, values) -> list\n"
@@ -715,7 +800,7 @@ PyMethodDef igraphmodule_EdgeSeq_methods[] = {
    "@param values: the new attribute values in a list\n"
   },
   {"select", (PyCFunction)igraphmodule_EdgeSeq_select,
-   METH_VARARGS | METH_KEYWORDS,
+   METH_VARARGS,
    "select(...) -> VertexSeq\n\n"
    "For internal use only.\n"
   },
@@ -810,8 +895,7 @@ PyGetSetDef igraphmodule_EdgeSeq_getseters[] = {
  */
 PyTypeObject igraphmodule_EdgeSeqType =
 {
-  PyObject_HEAD_INIT(NULL)                  /* */
-  0,                                        /* ob_size */
+  PyVarObject_HEAD_INIT(0, 0)
   "igraph.core.EdgeSeq",                    /* tp_name */
   sizeof(igraphmodule_EdgeSeqObject),       /* tp_basicsize */
   0,                                        /* tp_itemsize */
@@ -819,7 +903,7 @@ PyTypeObject igraphmodule_EdgeSeqType =
   0,                                        /* tp_print */
   0,                                        /* tp_getattr */
   0,                                        /* tp_setattr */
-  0,                                        /* tp_compare */
+  0,                                        /* tp_compare (2.x) / tp_reserved (3.x) */
   0,                                        /* tp_repr */
   0,                                        /* tp_as_number */
   &igraphmodule_EdgeSeq_as_sequence,        /* tp_as_sequence */
