@@ -412,6 +412,112 @@ PyObject* igraphmodule_compare_communities(PyObject *self,
 }
 
 
+PyObject* igraphmodule_is_degree_sequence(PyObject *self,
+  PyObject *args, PyObject *kwds) {
+  static char* kwlist[] = { "out_deg", "in_deg", NULL };
+  PyObject *out_deg_o = 0, *in_deg_o = 0;
+  igraph_vector_t out_deg, in_deg;
+  igraph_bool_t is_directed, result;
+
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O", kwlist,
+      &out_deg_o, &in_deg_o))
+    return NULL;
+
+  is_directed = (in_deg_o != 0 && in_deg_o != Py_None);
+
+  if (igraphmodule_PyObject_to_vector_t(out_deg_o, &out_deg, 0, 0))
+    return NULL;
+
+  if (is_directed && igraphmodule_PyObject_to_vector_t(in_deg_o, &in_deg, 0, 0)) {
+    igraph_vector_destroy(&out_deg);
+    return NULL;
+  }
+
+  if (igraph_is_degree_sequence(&out_deg, is_directed ? &in_deg : 0, &result)) {
+    igraphmodule_handle_igraph_error();
+    igraph_vector_destroy(&out_deg);
+    if (is_directed)
+      igraph_vector_destroy(&in_deg);
+    return NULL;
+  }
+
+  igraph_vector_destroy(&out_deg);
+  if (is_directed)
+    igraph_vector_destroy(&in_deg);
+
+  if (result)
+    Py_RETURN_TRUE;
+  else
+    Py_RETURN_FALSE;
+}
+
+
+PyObject* igraphmodule_is_graphical_degree_sequence(PyObject *self,
+  PyObject *args, PyObject *kwds) {
+  static char* kwlist[] = { "out_deg", "in_deg", NULL };
+  PyObject *out_deg_o = 0, *in_deg_o = 0;
+  igraph_vector_t out_deg, in_deg;
+  igraph_bool_t is_directed, result;
+
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O", kwlist,
+      &out_deg_o, &in_deg_o))
+    return NULL;
+
+  is_directed = (in_deg_o != 0 && in_deg_o != Py_None);
+
+  if (igraphmodule_PyObject_to_vector_t(out_deg_o, &out_deg, 0, 0))
+    return NULL;
+
+  if (is_directed && igraphmodule_PyObject_to_vector_t(in_deg_o, &in_deg, 0, 0)) {
+    igraph_vector_destroy(&out_deg);
+    return NULL;
+  }
+
+  if (igraph_is_graphical_degree_sequence(&out_deg, is_directed ? &in_deg : 0, &result)) {
+    igraphmodule_handle_igraph_error();
+    igraph_vector_destroy(&out_deg);
+    if (is_directed)
+      igraph_vector_destroy(&in_deg);
+    return NULL;
+  }
+
+  igraph_vector_destroy(&out_deg);
+  if (is_directed)
+    igraph_vector_destroy(&in_deg);
+
+  if (result)
+    Py_RETURN_TRUE;
+  else
+    Py_RETURN_FALSE;
+}
+
+
+PyObject* igraphmodule_power_law_fit(PyObject *self, PyObject *args, PyObject *kwds) {
+  static char* kwlist[] = { "data", "xmin", "force_continuous", NULL };
+  PyObject *data_o, *force_continuous_o = Py_False;
+  igraph_vector_t data;
+  igraph_plfit_result_t result;
+  double xmin = -1;
+
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|dO", kwlist, &data_o,
+        &xmin, &force_continuous_o))
+    return NULL;
+
+  if (igraphmodule_PyObject_float_to_vector_t(data_o, &data))
+    return NULL;
+
+  if (igraph_power_law_fit(&data, &result, xmin, PyObject_IsTrue(force_continuous_o))) {
+    igraphmodule_handle_igraph_error();
+    igraph_vector_destroy(&data);
+    return NULL;
+  }
+
+  igraph_vector_destroy(&data);
+
+  return Py_BuildValue("Oddddd", result.continuous ? Py_True : Py_False,
+      result.alpha, result.xmin, result.L, result.D, result.p);
+}
+
 PyObject* igraphmodule_split_join_distance(PyObject *self,
   PyObject *args, PyObject *kwds) {
   static char* kwlist[] = { "comm1", "comm2", NULL };
@@ -449,11 +555,15 @@ static PyMethodDef igraphmodule_methods[] =
 {
   {"community_to_membership", (PyCFunction)igraphmodule_community_to_membership,
     METH_VARARGS | METH_KEYWORDS,
-    "community_to_membership(merges, nodes, steps, return_csize=False)\n\n@undocumented"
+    "community_to_membership(merges, nodes, steps, return_csize=False)"
   },
   {"_compare_communities", (PyCFunction)igraphmodule_compare_communities,
     METH_VARARGS | METH_KEYWORDS,
-    "_compare_communities(comm1, comm2, method=\"vi\")\n\n@undocumented"
+    "_compare_communities(comm1, comm2, method=\"vi\")"
+  },
+  {"_power_law_fit", (PyCFunction)igraphmodule_power_law_fit,
+    METH_VARARGS | METH_KEYWORDS,
+    "_power_law_fit(data, xmin=-1, force_continuous=False)"
   },
   {"convex_hull", (PyCFunction)igraphmodule_convex_hull,
     METH_VARARGS | METH_KEYWORDS,
@@ -467,6 +577,42 @@ static PyMethodDef igraphmodule_methods[] =
     "  indices corresponding to them, depending on the C{coords}\n"
     "  parameter."
   },
+  {"is_degree_sequence", (PyCFunction)igraphmodule_is_degree_sequence,
+    METH_VARARGS | METH_KEYWORDS,
+    "is_degree_sequence(out_deg, in_deg=None)\n\n"
+    "Returns whether a list of degrees can be a degree sequence of some graph.\n\n"
+    "Note that it is not required for the graph to be simple; in other words,\n"
+    "this function may return C{True} for degree sequences that can be realized\n"
+    "using one or more multiple or loop edges only.\n\n"
+    "In particular, this function checks whether\n\n"
+    "  - all the degrees are non-negative\n"
+    "  - for undirected graphs, the sum of degrees are even\n"
+    "  - for directed graphs, the two degree sequences are of the same length and\n"
+    "    equal sums\n\n"
+    "@param out_deg: the list of degrees. For directed graphs, this list must\n"
+    "  contain the out-degrees of the vertices.\n"
+    "@param in_deg: the list of in-degrees for directed graphs. This parameter\n"
+    "  must be C{None} for undirected graphs.\n"
+    "@return: C{True} if there exists some graph that can realize the given degree\n"
+    "  sequence, C{False} otherwise."
+    "@see: L{is_graphical_degree_sequence()} if you do not want to allow multiple\n"
+    "  or loop edges.\n"
+  },
+  {"is_graphical_degree_sequence", (PyCFunction)igraphmodule_is_graphical_degree_sequence,
+    METH_VARARGS | METH_KEYWORDS,
+    "is_graphical_degree_sequence(out_deg, in_deg=None)\n\n"
+    "Returns whether a list of degrees can be a degree sequence of some simple graph.\n\n"
+    "Note that it is required for the graph to be simple; in other words,\n"
+    "this function will return C{False} for degree sequences that cannot be realized\n"
+    "without using one or more multiple or loop edges.\n\n"
+    "@param out_deg: the list of degrees. For directed graphs, this list must\n"
+    "  contain the out-degrees of the vertices.\n"
+    "@param in_deg: the list of in-degrees for directed graphs. This parameter\n"
+    "  must be C{None} for undirected graphs.\n"
+    "@return: C{True} if there exists some simple graph that can realize the given\n"
+    "  degree sequence, C{False} otherwise.\n"
+    "@see: L{is_degree_sequence()} if you want to allow multiple or loop edges.\n"
+  },
   {"set_progress_handler", igraphmodule_set_progress_handler, METH_O,
       "set_progress_handler(handler)\n\n"
       "Sets the handler to be called when igraph is performing a long operation.\n"
@@ -502,11 +648,17 @@ static PyMethodDef igraphmodule_methods[] =
   },
   {"_split_join_distance", (PyCFunction)igraphmodule_split_join_distance,
     METH_VARARGS | METH_KEYWORDS,
-    "_split_join_distance(comm1, comm2)\n\n@undocumented"
+    "_split_join_distance(comm1, comm2)"
   },
   {NULL, NULL, 0, NULL}
 };
 
+#define MODULE_DOCS \
+  "Low-level Python interface for the igraph library. " \
+  "Should not be used directly.\n\n"                    \
+  "@undocumented: community_to_membership, _compare_communities, _power_law_fit, " \
+  "_split_join_distance"
+
 /**
  * Module definition table (only for Python 3.x)
  */
@@ -514,8 +666,7 @@ static PyMethodDef igraphmodule_methods[] =
 static struct PyModuleDef moduledef = {
   PyModuleDef_HEAD_INIT,
   "igraph._igraph",                   /* m_name */
-  "Low-level Python interface for the igraph library. "
-  "Should not be used directly.",     /* m_doc */
+  MODULE_DOCS,                        /* m_doc */
   sizeof(struct module_state),        /* m_size */
   igraphmodule_methods,               /* m_methods */
   0,                                  /* m_reload */
@@ -620,9 +771,7 @@ extern PyObject* igraphmodule_arpack_options_default;
 #ifdef IGRAPH_PYTHON3
   m = PyModule_Create(&moduledef);
 #else
-  m = Py_InitModule3("igraph._igraph", igraphmodule_methods,
-		     "Low-level Python interface for the igraph library. "
-		     "Should not be used directly.");
+  m = Py_InitModule3("igraph._igraph", igraphmodule_methods, MODULE_DOCS);
 #endif
 
   if (m == NULL)
@@ -671,6 +820,7 @@ extern PyObject* igraphmodule_arpack_options_default;
   PyModule_AddIntConstant(m, "GET_ADJACENCY_BOTH", IGRAPH_GET_ADJACENCY_BOTH);
 
   PyModule_AddIntConstant(m, "REWIRING_SIMPLE", IGRAPH_REWIRING_SIMPLE);
+  PyModule_AddIntConstant(m, "REWIRING_SIMPLE_LOOPS", IGRAPH_REWIRING_SIMPLE_LOOPS);
 
   PyModule_AddIntConstant(m, "ADJ_DIRECTED", IGRAPH_ADJ_DIRECTED);
   PyModule_AddIntConstant(m, "ADJ_UNDIRECTED", IGRAPH_ADJ_UNDIRECTED);
