@@ -3,8 +3,7 @@
 
 /* 
    IGraph library.
-   Copyright (C) 2006  Gabor Csardi <csardi@rmki.kfki.hu>
-   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+   Copyright (C) 2006-2012  Tamas Nepusz <ntamas@gmail.com>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,11 +22,14 @@
 
 */
 
-#include "vertexseqobject.h"
-#include "vertexobject.h"
+#include <Python.h>
+#include "attributes.h"
 #include "common.h"
 #include "convert.h"
 #include "error.h"
+#include "py2compat.h"
+#include "vertexseqobject.h"
+#include "vertexobject.h"
 
 #define GET_GRAPH(obj) (((igraphmodule_GraphObject*)obj->gref)->g)
 
@@ -68,7 +70,7 @@ igraphmodule_VertexSeqObject*
 igraphmodule_VertexSeq_copy(igraphmodule_VertexSeqObject* o) {
   igraphmodule_VertexSeqObject *copy;
 
-  copy=(igraphmodule_VertexSeqObject*)PyType_GenericNew(o->ob_type, 0, 0);
+  copy=(igraphmodule_VertexSeqObject*)PyType_GenericNew(Py_TYPE(o), 0, 0);
   if (copy == NULL) return NULL;
  
   if (igraph_vs_type(&o->vs) == IGRAPH_VS_VECTOR) {
@@ -114,12 +116,12 @@ int igraphmodule_VertexSeq_init(igraphmodule_VertexSeqObject *self,
     igraph_vs_all(&vs);
   } else if (PyInt_Check(vsobj)) {
     /* We selected a single vertex */
-    long idx = PyInt_AsLong(vsobj);
+    long int idx = PyInt_AsLong(vsobj);
     if (idx < 0 || idx >= igraph_vcount(&((igraphmodule_GraphObject*)g)->g)) {
-      PyErr_SetString(PyExc_ValueError, "vertex index out of bounds");
+      PyErr_SetString(PyExc_ValueError, "vertex index out of range");
       return -1;
     }
-    igraph_vs_1(&vs, idx);
+    igraph_vs_1(&vs, (igraph_integer_t)idx);
   } else {
     igraph_vector_t v;
     igraph_integer_t n = igraph_vcount(&((igraphmodule_GraphObject*)g)->g);
@@ -127,7 +129,7 @@ int igraphmodule_VertexSeq_init(igraphmodule_VertexSeqObject *self,
       return -1;
     if (!igraph_vector_isininterval(&v, 0, n-1)) {
       igraph_vector_destroy(&v);
-      PyErr_SetString(PyExc_ValueError, "vertex index out of bounds");
+      PyErr_SetString(PyExc_ValueError, "vertex index out of range");
       return -1;
     }
     if (igraph_vs_vector_copy(&vs, &v)) {
@@ -157,7 +159,7 @@ void igraphmodule_VertexSeq_dealloc(igraphmodule_VertexSeqObject* self) {
     Py_DECREF(self->gref);
     self->gref=0;
   }
-  self->ob_type->tp_free((PyObject*)self);
+  Py_TYPE(self)->tp_free((PyObject*)self);
   RC_DEALLOC("VertexSeq", self);
 }
 
@@ -184,25 +186,27 @@ int igraphmodule_VertexSeq_sq_length(igraphmodule_VertexSeqObject* self) {
 PyObject* igraphmodule_VertexSeq_sq_item(igraphmodule_VertexSeqObject* self,
                      Py_ssize_t i) {
   igraph_t *g;
-  long idx = -1;
+  igraph_integer_t idx = -1;
 
   if (!self->gref) return NULL;
   g=&GET_GRAPH(self);
   switch (igraph_vs_type(&self->vs)) {
     case IGRAPH_VS_ALL:
-      if (i >= 0 && i < (long)igraph_vcount(g)) idx = i;
+      if (i >= 0 && i < igraph_vcount(g))
+        idx = (igraph_integer_t)i;
       break;
     case IGRAPH_VS_VECTOR:
     case IGRAPH_VS_VECTORPTR:
       if (i >= 0 && i < igraph_vector_size(self->vs.data.vecptr))
-        idx = (long)VECTOR(*self->vs.data.vecptr)[i];
+        idx = (igraph_integer_t)VECTOR(*self->vs.data.vecptr)[i];
       break;
     case IGRAPH_VS_1:
-      if (i == 0) idx = (long)self->vs.data.vid;
+      if (i == 0)
+        idx = self->vs.data.vid;
       break;
     case IGRAPH_VS_SEQ:
       if (i >= 0 && i < self->vs.data.seq.to - self->vs.data.seq.from)
-        idx = (long)(self->vs.data.seq.from + i);
+        idx = self->vs.data.seq.from + (igraph_integer_t)i;
       break;
     /* TODO: IGRAPH_VS_ADJ, IGRAPH_VS_NONADJ - someday :) They are unused
        yet in the Python interface */
@@ -223,20 +227,6 @@ PyObject* igraphmodule_VertexSeq_attribute_names(igraphmodule_VertexSeqObject* s
   return igraphmodule_Graph_vertex_attributes(self->gref);
 }
 
-/** \ingroup python_interface_vertexseq
- * \brief Returns the count of attribute names
- */
-PyObject* igraphmodule_VertexSeq_attribute_count(igraphmodule_VertexSeqObject* self) {
-  PyObject *list;
-  long int size;
-  
-  list=igraphmodule_Graph_vertex_attributes(self->gref);
-  size=PySequence_Size(list);
-  Py_DECREF(list);
-
-  return Py_BuildValue("i", size);
-}
-
 /** \ingroup python_interface_vertexseq
  * \brief Returns the list of values for a given attribute
  */
@@ -246,7 +236,7 @@ PyObject* igraphmodule_VertexSeq_get_attribute_values(igraphmodule_VertexSeqObje
   long int i, n;
 
   PyErr_Clear();
-  values=PyDict_GetItem(((PyObject**)gr->g.attr)[ATTRHASH_IDX_VERTEX], o);
+  values=PyDict_GetItem(ATTR_STRUCT_DICT(&gr->g)[ATTRHASH_IDX_VERTEX], o);
   if (!values) {
     PyErr_SetString(PyExc_KeyError, "Attribute does not exist");
     return NULL;
@@ -308,7 +298,7 @@ PyObject* igraphmodule_VertexSeq_get_attribute_values_mapping(igraphmodule_Verte
     return igraphmodule_VertexSeq_sq_item(self, PyInt_AsLong(o));
 
   /* Handle strings according to the mapping protocol */
-  if (PyString_Check(o) || PyUnicode_Check(o))
+  if (PyBaseString_Check(o))
     return igraphmodule_VertexSeq_get_attribute_values(self, o);
 
   /* Handle iterables and slices by calling the select() method */
@@ -317,7 +307,7 @@ PyObject* igraphmodule_VertexSeq_get_attribute_values_mapping(igraphmodule_Verte
     args = Py_BuildValue("(O)", o);
     if (!args)
       return NULL;
-    result = igraphmodule_VertexSeq_select(self, args, NULL);
+    result = igraphmodule_VertexSeq_select(self, args);
     Py_DECREF(args);
     return result;
   }
@@ -333,11 +323,14 @@ int igraphmodule_VertexSeq_set_attribute_values_mapping(igraphmodule_VertexSeqOb
   PyObject *dict, *list, *item;
   igraphmodule_GraphObject *gr;
   igraph_vector_t vs;
-  long i, n;
+  long i, j, n, no_of_nodes;
 
   gr = self->gref;
-  dict = ((PyObject**)gr->g.attr)[ATTRHASH_IDX_VERTEX];
-  
+  dict = ATTR_STRUCT_DICT(&gr->g)[ATTRHASH_IDX_VERTEX];
+
+  if (PyString_IsEqualToASCIIString(attrname, "name"))
+    igraphmodule_invalidate_vertex_name_index(&gr->g);
+
   if (values == 0) {
     if (igraph_vs_type(&self->vs) == IGRAPH_VS_ALL)
       return PyDict_DelItem(dict, attrname);
@@ -345,12 +338,26 @@ int igraphmodule_VertexSeq_set_attribute_values_mapping(igraphmodule_VertexSeqOb
     return -1;
   }
 
+ if (PyString_Check(values) || !PySequence_Check(values)) {
+    /* If values is a string or not a sequence, we construct a list with a
+     * single element (the value itself) and then call ourselves again */
+    int result;
+    PyObject *newList = PyList_New(1);
+    if (newList == 0) return -1;
+    Py_INCREF(values);
+    PyList_SET_ITEM(newList, 0, values);    /* reference stolen here */
+    result = igraphmodule_VertexSeq_set_attribute_values_mapping(self, attrname, newList);
+    Py_DECREF(newList);
+    return result;
+  }
+
   n=PySequence_Size(values);
   if (n<0) return -1;
 
   if (igraph_vs_type(&self->vs) == IGRAPH_VS_ALL) {
-    if (n != (long)igraph_vcount(&gr->g)) {
-      PyErr_SetString(PyExc_ValueError, "value list length must be equal to the number of vertices in the vertex sequence");
+    no_of_nodes = (long)igraph_vcount(&gr->g);
+    if (n == 0 && no_of_nodes > 0) {
+      PyErr_SetString(PyExc_ValueError, "sequence must not be empty");
       return -1;
     }
 
@@ -358,22 +365,24 @@ int igraphmodule_VertexSeq_set_attribute_values_mapping(igraphmodule_VertexSeqOb
     list = PyDict_GetItem(dict, attrname);
     if (list != 0) {
       /* Yes, we have. Modify its items to the items found in values */
-      for (i=0; i<n; i++) {
-        item = PySequence_GetItem(values, i);
+      for (i=0, j=0; i<no_of_nodes; i++, j++) {
+        if (j == n) j = 0;
+        item = PySequence_GetItem(values, j);
         if (item == 0) return -1;
         /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         if (PyList_SetItem(list, i, item)) {
           Py_DECREF(item);
           return -1;
-        } /* PyList_SetItem stole a reference to the item automatically */  
+        } /* PyList_SetItem stole a reference to the item automatically */
       }
     } else if (values != 0) {
       /* We don't have attributes with the given name yet. Create an entry
        * in the dict, create a new list and copy everything */
-      list = PyList_New(n);
+      list = PyList_New(no_of_nodes);
       if (list == 0) return -1;
-      for (i=0; i<n; i++) {
-        item = PySequence_GetItem(values, i);
+      for (i=0, j=0; i<no_of_nodes; i++, j++) {
+        if (j == n) j = 0;
+        item = PySequence_GetItem(values, j);
         if (item == 0) { Py_DECREF(list); return -1; }
         /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         PyList_SET_ITEM(list, i, item);
@@ -397,8 +406,9 @@ int igraphmodule_VertexSeq_set_attribute_values_mapping(igraphmodule_VertexSeqOb
       igraph_vector_destroy(&vs);
       return -1;
     }
-    if (n != (long)igraph_vector_size(&vs)) {
-      PyErr_SetString(PyExc_ValueError, "value list length must be equal to the number of vertices in the vertex sequence");
+    no_of_nodes = (long)igraph_vector_size(&vs);
+    if (n == 0 && no_of_nodes > 0) {
+      PyErr_SetString(PyExc_ValueError, "sequence must not be empty");
       igraph_vector_destroy(&vs);
       return -1;
     }
@@ -406,8 +416,9 @@ int igraphmodule_VertexSeq_set_attribute_values_mapping(igraphmodule_VertexSeqOb
     list = PyDict_GetItem(dict, attrname);
     if (list != 0) {
       /* Yes, we have. Modify its items to the items found in values */
-      for (i=0; i<n; i++) {
-        item = PySequence_GetItem(values, i);
+      for (i=0, j=0; i<no_of_nodes; i++, j++) {
+        if (j == n) j = 0;
+        item = PySequence_GetItem(values, j);
         if (item == 0) {
           igraph_vector_destroy(&vs);
           return -1;
@@ -434,8 +445,9 @@ int igraphmodule_VertexSeq_set_attribute_values_mapping(igraphmodule_VertexSeqOb
         Py_INCREF(Py_None);
         PyList_SET_ITEM(list, i, Py_None);
       }
-      for (i=0; i<n; i++) {
-        item = PySequence_GetItem(values, i);
+      for (i=0, j=0; i<no_of_nodes; i++, j++) {
+        if (j == n) j = 0;
+        item = PySequence_GetItem(values, j);
         if (item == 0) {
           igraph_vector_destroy(&vs);
           Py_DECREF(list); return -1;
@@ -470,19 +482,91 @@ PyObject* igraphmodule_VertexSeq_set_attribute_values(igraphmodule_VertexSeqObje
   Py_RETURN_NONE;
 }
 
+/**
+ * \ingroup python_interface_vertexseq
+ * \brief Selects a single vertex from the vertex sequence based on some criteria
+ */
+PyObject* igraphmodule_VertexSeq_find(igraphmodule_VertexSeqObject *self, PyObject *args) {
+  PyObject *item;
+  igraph_integer_t i;
+  Py_ssize_t n;
+  igraph_vit_t vit;
+
+  if (!PyArg_ParseTuple(args, "O", &item))
+    return NULL;
+
+  if (PyCallable_Check(item)) {
+    /* Call the callable for every vertex in the current sequence and return
+     * the first one for which it evaluates to True */
+    n = PySequence_Size((PyObject*)self);
+    for (i=0; i<n; i++) {
+      PyObject *vertex = PySequence_GetItem((PyObject*)self, i);
+      PyObject *call_result;
+      if (vertex == 0)
+        return NULL;
+      call_result = PyObject_CallFunctionObjArgs(item, vertex, NULL);
+      if (call_result == 0) {
+        Py_DECREF(vertex);
+        return NULL;
+      }
+      if (PyObject_IsTrue(call_result)) {
+        Py_DECREF(call_result);
+        return vertex;  /* reference passed to caller */
+      }
+      Py_DECREF(call_result);
+      Py_DECREF(vertex);
+    }
+  } else if (PyInt_Check(item)) {
+    /* Integers are interpreted as indices on the vertex set and NOT on the
+     * original, untouched vertex sequence of the graph */
+    return PySequence_GetItem((PyObject*)self, PyInt_AsLong(item));
+  } else if (PyBaseString_Check(item)) {
+    /* Strings are interpreted as vertex names */
+    if (igraphmodule_get_vertex_id_by_name(&self->gref->g, item, &i))
+      return NULL;
+
+    /* We now have the ID of the vertex in the graph. If the vertex sequence
+     * itself represents the full vertex sequence of the graph, we can return
+     * here. If not, we have to check whether the vertex sequence contains this
+     * ID or not. */
+    if (igraph_vs_is_all(&self->vs))
+      return PySequence_GetItem((PyObject*)self, i);
+
+    if (igraph_vit_create(&self->gref->g, self->vs, &vit)) {
+      igraphmodule_handle_igraph_error();
+      return NULL;
+    }
+
+    for (n = 0; !IGRAPH_VIT_END(vit); IGRAPH_VIT_NEXT(vit), n++) {
+      if (IGRAPH_VIT_GET(vit) == i) {
+        igraph_vit_destroy(&vit);
+        return PySequence_GetItem((PyObject*)self, n);
+      }
+    }
+
+    igraph_vit_destroy(&vit);
+    PyErr_SetString(PyExc_ValueError, "vertex with the given name exists but not in the current sequence");
+    return NULL;
+  }
+
+  PyErr_SetString(PyExc_IndexError, "no such vertex");
+  return NULL;
+}
+
 /**
  * \ingroup python_interface_vertexseq
  * \brief Selects a subset of the vertex sequence based on some criteria
  */
 PyObject* igraphmodule_VertexSeq_select(igraphmodule_VertexSeqObject *self,
-  PyObject *args, PyObject *kwds) {
+  PyObject *args) {
   igraphmodule_VertexSeqObject *result;
   igraphmodule_GraphObject *gr;
   long i, j, n, m;
 
   gr=self->gref;
   result=igraphmodule_VertexSeq_copy(self);
-  if (result==0) return NULL;
+  if (result==0)
+    return NULL;
 
   /* First, filter by positional arguments */
   n = PyTuple_Size(args);
@@ -563,6 +647,7 @@ PyObject* igraphmodule_VertexSeq_select(igraphmodule_VertexSeqObject *self,
         igraphmodule_handle_igraph_error();
         return 0;
       }
+      m = igraph_vector_size(&v2);
       for (; i<n; i++) {
         PyObject *item2 = PyTuple_GET_ITEM(args, i);
         long idx;
@@ -574,6 +659,12 @@ PyObject* igraphmodule_VertexSeq_select(igraphmodule_VertexSeqObject *self,
           return NULL;
         }
         idx = PyInt_AsLong(item2);
+        if (idx >= m || idx < 0) {
+          PyErr_SetString(PyExc_ValueError, "vertex index out of range");
+          igraph_vector_destroy(&v);
+          igraph_vector_destroy(&v2);
+          return NULL;
+        }
         if (igraph_vector_push_back(&v, VECTOR(v2)[idx])) {
           Py_DECREF(result);
           igraphmodule_handle_igraph_error();
@@ -615,6 +706,7 @@ PyObject* igraphmodule_VertexSeq_select(igraphmodule_VertexSeqObject *self,
         igraphmodule_handle_igraph_error();
         return 0;
       }
+      m = igraph_vector_size(&v2);
 
       /* Create an appropriate iterator */
       if (PySlice_Check(item)) {
@@ -623,7 +715,9 @@ PyObject* igraphmodule_VertexSeq_select(igraphmodule_VertexSeqObject *self,
         PyObject* range;
         igraph_bool_t ok;
 
-        ok = (PySlice_GetIndicesEx((PySliceObject*)item, igraph_vector_size(&v2),
+        /* Casting to void* because Python 2.x expects PySliceObject*
+         * but Python 3.x expects PyObject* */
+        ok = (PySlice_GetIndicesEx((void*)item, igraph_vector_size(&v2),
               &start, &stop, &step, &sl) == 0);
         if (ok) {
           range = PyObject_CallFunction((PyObject*)&PyRange_Type, "lll", start, stop, step);
@@ -659,6 +753,14 @@ PyObject* igraphmodule_VertexSeq_select(igraphmodule_VertexSeqObject *self,
         if (PyInt_Check(item2)) {
           long idx = PyInt_AsLong(item2);
           Py_DECREF(item2);
+          if (idx >= m || idx < 0) {
+            PyErr_SetString(PyExc_ValueError, "vertex index out of range");
+            Py_DECREF(result);
+            Py_DECREF(iter);
+            igraph_vector_destroy(&v);
+            igraph_vector_destroy(&v2);
+            return NULL;
+          }
           if (igraph_vector_push_back(&v, VECTOR(v2)[idx])) {
             Py_DECREF(result);
             Py_DECREF(iter);
@@ -741,6 +843,29 @@ PyObject* igraphmodule_VertexSeq_get_indices(igraphmodule_VertexSeqObject* self,
   return result;
 }
 
+/**
+ * \ingroup python_interface_vertexseq
+ * Returns the internal dictionary mapping vertex names to vertex IDs.
+ */
+PyObject* igraphmodule_VertexSeq__name_index(igraphmodule_VertexSeqObject* self,
+  void* closure) {
+  igraphmodule_GraphObject *gr = self->gref;
+  PyObject* result = ATTR_NAME_INDEX(&gr->g);
+  if (result == 0)
+    Py_RETURN_NONE;
+  Py_INCREF(result);
+  return result;
+}
+
+/**
+ * \ingroup python_interface_vertexseq
+ * Re-creates the dictionary that maps vertex names to vertex IDs.
+ */
+PyObject* igraphmodule_VertexSeq__reindex_names(igraphmodule_VertexSeqObject* self) {
+  igraphmodule_index_vertex_names(&self->gref->g, 1);
+  Py_RETURN_NONE;
+}
+
 /**
  * \ingroup python_interface_vertexseq
  * Method table for the \c igraph.VertexSeq object
@@ -751,6 +876,11 @@ PyMethodDef igraphmodule_VertexSeq_methods[] = {
    "attribute_names() -> list\n\n"
    "Returns the attribute name list of the graph's vertices\n"
   },
+  {"find", (PyCFunction)igraphmodule_VertexSeq_find,
+   METH_VARARGS,
+   "find(condition) -> Vertex\n\n"
+   "For internal use only.\n"
+  },
   {"get_attribute_values", (PyCFunction)igraphmodule_VertexSeq_get_attribute_values,
    METH_O,
    "get_attribute_values(attrname) -> list\n"
@@ -770,10 +900,14 @@ PyMethodDef igraphmodule_VertexSeq_methods[] = {
    "@param values: the new attribute values in a list\n"
   },
   {"select", (PyCFunction)igraphmodule_VertexSeq_select,
-   METH_VARARGS | METH_KEYWORDS,
+   METH_VARARGS,
    "select(...) -> VertexSeq\n\n"
    "For internal use only.\n"
   },
+  {"_reindex_names", (PyCFunction)igraphmodule_VertexSeq__reindex_names, METH_NOARGS,
+   "Re-creates the dictionary that maps vertex names to IDs.\n\n"
+   "For internal use only.\n"
+  },
   {NULL}
 };
 
@@ -821,6 +955,9 @@ PyGetSetDef igraphmodule_VertexSeq_getseters[] = {
   {"indices", (getter)igraphmodule_VertexSeq_get_indices, NULL,
       "The vertex indices in this vertex sequence", NULL,
   },
+  {"_name_index", (getter)igraphmodule_VertexSeq__name_index, NULL,
+      "The internal index mapping vertex names to IDs", NULL
+  },
   {NULL}
 };
 
@@ -830,8 +967,7 @@ PyGetSetDef igraphmodule_VertexSeq_getseters[] = {
  */
 PyTypeObject igraphmodule_VertexSeqType =
 {
-  PyObject_HEAD_INIT(NULL)                    /* */
-  0,                                          /* ob_size */
+  PyVarObject_HEAD_INIT(0, 0)
   "igraph.core.VertexSeq",                    /* tp_name */
   sizeof(igraphmodule_VertexSeqObject),       /* tp_basicsize */
   0,                                          /* tp_itemsize */
@@ -839,7 +975,7 @@ PyTypeObject igraphmodule_VertexSeqType =
   0,                                          /* tp_print */
   0,                                          /* tp_getattr */
   0,                                          /* tp_setattr */
-  0,                                          /* tp_compare */
+  0,                                          /* tp_compare (2.x) / tp_reserved (3.x) */
   0,                                          /* tp_repr */
   0,                                          /* tp_as_number */
   &igraphmodule_VertexSeq_as_sequence,        /* tp_as_sequence */
